#  -*- coding: utf-8 -*-
""" Set of ESC/POS Commands (Constants)

This module contains constants that are described in the esc/pos-documentation.
Since there is no definitive and unified specification for all esc/pos-like printers the constants could later be
moved to `capabilities` as in `escpos-php by @mike42 <https://github.com/mike42/escpos-php>`_.

:author: `Manuel F Martinez <manpaz@bashlinux.com>`_ and others
:organization: Bashlinux and `python-escpos <https://github.com/python-escpos>`_
:copyright: Copyright (c) 2012 Bashlinux
:license: GNU GPL v3
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import six

# Control characters
# as labelled in http://www.novopos.ch/client/EPSON/TM-T20/TM-T20_eng_qr.pdf
NUL = b'\x00'
EOT = b'\x04'
ENQ = b'\x05'
DLE = b'\x10'
DC4 = b'\x14'
CAN = b'\x18'
ESC = b'\x1b'
FS  = b'\x1c'
GS  = b'\x1d'

# Feed control sequences
CTL_LF = b'\n'              # Print and line feed
CTL_FF = b'\f'              # Form feed
CTL_CR = b'\r'              # Carriage return
CTL_HT = b'\t'              # Horizontal tab
CTL_SET_HT = ESC + b'\x44'  # Set horizontal tab positions
CTL_VT = b'\v'              # Vertical tab

# Printer hardware
HW_INIT   = ESC + b'@'             # Clear data in buffer and reset modes
HW_SELECT = ESC + b'=\x01'         # Printer select

HW_RESET  = ESC + b'\x3f\x0a\x00'   # Reset printer hardware
                                    # (TODO: Where is this specified?)

# Cash Drawer (ESC p <pin> <on time: 2*ms> <off time: 2*ms>)
_CASH_DRAWER = lambda m, t1='', t2='': ESC + b'p' + m + six.int2byte(t1) + six.int2byte(t2)
CD_KICK_2 = _CASH_DRAWER(b'\x00', 50, 50)  # Sends a pulse to pin 2 []
CD_KICK_5 = _CASH_DRAWER(b'\x01', 50, 50)  # Sends a pulse to pin 5 []

# Paper Cutter
_CUT_PAPER = lambda m: GS + b'V' + m
PAPER_FULL_CUT = _CUT_PAPER(b'\x00')  # Full cut paper
PAPER_PART_CUT = _CUT_PAPER(b'\x01')  # Partial cut paper
PAPER_FEED_CUT = _CUT_PAPER(b'\x01')  # Partial cut paper

# Beep
BEEP = b'\x07'

# Panel buttons (e.g. the FEED button)
_PANEL_BUTTON = lambda n: ESC + b'c5' + six.int2byte(n)
PANEL_BUTTON_ON = _PANEL_BUTTON(0)  # enable all panel buttons
PANEL_BUTTON_OFF = _PANEL_BUTTON(1)  # disable all panel buttons

# Sheet modes
SHEET_SLIP_MODE = ESC + b'\x63\x30\x04'  # slip paper
SHEET_ROLL_MODE = ESC + b'\x63\x30\x01'  # paper roll

# Text format
# TODO: Acquire the "ESC/POS Application Programming Guide for Paper Roll
#       Printers" and tidy up this stuff too.
TXT_FLIP_ON    = ESC + b'\x7b\x01'
TXT_FLIP_OFF   = ESC + b'\x7b\x00'
TXT_SMOOTH_ON  = GS + b'\x62\x01'
TXT_SMOOTH_OFF = GS + b'\x62\x00'
TXT_SIZE       = GS + b'!'
TXT_WIDTH      = {1: 0x00,
                  2: 0x10,
                  3: 0x20,
                  4: 0x30,
                  5: 0x40,
                  6: 0x50,
                  7: 0x60,
                  8: 0x70}
TXT_HEIGHT     = {1: 0x00,
                  2: 0x01,
                  3: 0x02,
                  4: 0x03,
                  5: 0x04,
                  6: 0x05,
                  7: 0x06,
                  8: 0x07}
TXT_NORMAL     = ESC + b'!\x00'     # Normal text
TXT_2HEIGHT    = ESC + b'!\x10'     # Double height text
TXT_2WIDTH     = ESC + b'!\x20'     # Double width text
TXT_4SQUARE    = ESC + b'!\x30'     # Quad area text
TXT_UNDERL_OFF = ESC + b'\x2d\x00'  # Underline font OFF
TXT_UNDERL_ON  = ESC + b'\x2d\x01'  # Underline font 1-dot ON
TXT_UNDERL2_ON = ESC + b'\x2d\x02'  # Underline font 2-dot ON
TXT_BOLD_OFF   = ESC + b'\x45\x00'  # Bold font OFF
TXT_BOLD_ON    = ESC + b'\x45\x01'  # Bold font ON
TXT_ALIGN_LT   = ESC + b'\x61\x00'  # Left justification
TXT_ALIGN_CT   = ESC + b'\x61\x01'  # Centering
TXT_ALIGN_RT   = ESC + b'\x61\x02'  # Right justification
TXT_INVERT_ON  = GS  + b'\x42\x01'  # Inverse Printing ON
TXT_INVERT_OFF = GS  + b'\x42\x00'  # Inverse Printing OFF

# Fonts
SET_FONT = lambda n: ESC + b'\x4d' + n
TXT_FONT_A     = SET_FONT(b'\x00')  # Font type A
TXT_FONT_B     = SET_FONT(b'\x01')  # Font type B

# Text colors
TXT_COLOR_BLACK = ESC + b'\x72\x00'  # Default Color
TXT_COLOR_RED = ESC + b'\x72\x01'    # Alternative Color (Usually Red)

# Spacing
LINESPACING_RESET = ESC + b'2'
LINESPACING_FUNCS = {
  60: ESC + b'A',  # line_spacing/60 of an inch, 0 <= line_spacing <= 85
  360: ESC + b'+', # line_spacing/360 of an inch, 0 <= line_spacing <= 255
  180: ESC + b'3', # line_spacing/180 of an inch, 0 <= line_spacing <= 255
  1: ESC + b'3' + b'('
}

# Char code table
CHARCODE_PC437  = ESC + b'\x74\x00'  # USA: Standard Europe
CHARCODE_JIS    = ESC + b'\x74\x01'  # Japanese Katakana
CHARCODE_PC850  = ESC + b'\x74\x02'  # Multilingual
CHARCODE_PC860  = ESC + b'\x74\x03'  # Portuguese
CHARCODE_PC863  = ESC + b'\x74\x04'  # Canadian-French
CHARCODE_PC865  = ESC + b'\x74\x05'  # Nordic
CHARCODE_WEU    = ESC + b'\x74\x06'  # Simplified Kanji, Hirakana
CHARCODE_GREEK  = ESC + b'\x74\x07'  # Simplified Kanji
CHARCODE_HEBREW = ESC + b'\x74\x08'  # Simplified Kanji
CHARCODE_PC1252 = ESC + b'\x74\x11'  # Western European Windows Code Set
CHARCODE_PC866  = ESC + b'\x74\x12'  # Cirillic #2
CHARCODE_PC852  = ESC + b'\x74\x13'  # Latin 2
CHARCODE_PC858  = ESC + b'\x74\x14'  # Euro
CHARCODE_THAI42 = ESC + b'\x74\x15'  # Thai character code 42
CHARCODE_THAI11 = ESC + b'\x74\x16'  # Thai character code 11
CHARCODE_THAI13 = ESC + b'\x74\x17'  # Thai character code 13
CHARCODE_THAI14 = ESC + b'\x74\x18'  # Thai character code 14
CHARCODE_THAI16 = ESC + b'\x74\x19'  # Thai character code 16
CHARCODE_THAI17 = ESC + b'\x74\x1a'  # Thai character code 17
CHARCODE_THAI18 = ESC + b'\x74\x1b'  # Thai character code 18

# Barcode format
_SET_BARCODE_TXT_POS = lambda n: GS + b'H' + n
BARCODE_TXT_OFF = _SET_BARCODE_TXT_POS(b'\x00')  # HRI barcode chars OFF
BARCODE_TXT_ABV = _SET_BARCODE_TXT_POS(b'\x01')  # HRI barcode chars above
BARCODE_TXT_BLW = _SET_BARCODE_TXT_POS(b'\x02')  # HRI barcode chars below
BARCODE_TXT_BTH = _SET_BARCODE_TXT_POS(b'\x03')  # HRI both above and below

_SET_HRI_FONT = lambda n: GS + b'f' + n
BARCODE_FONT_A = _SET_HRI_FONT(b'\x00')  # Font type A for HRI barcode chars
BARCODE_FONT_B = _SET_HRI_FONT(b'\x01')  # Font type B for HRI barcode chars

BARCODE_HEIGHT = GS + b'h'  # Barcode Height [1-255]
BARCODE_WIDTH  = GS + b'w'  # Barcode Width  [2-6]

# NOTE: This isn't actually an ESC/POS command. It's the common prefix to the
#      two "print bar code" commands:
#      -  Type A: "GS k <type as integer> <data> NUL"
#      -  TYPE B: "GS k <type as letter> <data length> <data>"
#      The latter command supports more barcode types
_SET_BARCODE_TYPE = lambda m: GS + b'k' + six.int2byte(m)

# Barcodes for printing function type A
BARCODE_TYPE_A = {
    'UPC-A':   _SET_BARCODE_TYPE(0),
    'UPC-E':   _SET_BARCODE_TYPE(1),
    'EAN13':   _SET_BARCODE_TYPE(2),
    'EAN8':    _SET_BARCODE_TYPE(3),
    'CODE39':  _SET_BARCODE_TYPE(4),
    'ITF':     _SET_BARCODE_TYPE(5),
    'NW7':     _SET_BARCODE_TYPE(6),
    'CODABAR': _SET_BARCODE_TYPE(6),  # Same as NW7
}

# Barcodes for printing function type B
# The first 8 are the same barcodes as type A
BARCODE_TYPE_B = {
    'UPC-A':                       _SET_BARCODE_TYPE(65),
    'UPC-E':                       _SET_BARCODE_TYPE(66),
    'EAN13':                       _SET_BARCODE_TYPE(67),
    'EAN8':                        _SET_BARCODE_TYPE(68),
    'CODE39':                      _SET_BARCODE_TYPE(69),
    'ITF':                         _SET_BARCODE_TYPE(70),
    'NW7':                         _SET_BARCODE_TYPE(71),
    'CODABAR':                     _SET_BARCODE_TYPE(71),  # Same as NW7
    'CODE93':                      _SET_BARCODE_TYPE(72),
    'CODE128':                     _SET_BARCODE_TYPE(73),
    'GS1-128':                     _SET_BARCODE_TYPE(74),
    'GS1 DATABAR OMNIDIRECTIONAL': _SET_BARCODE_TYPE(75),
    'GS1 DATABAR TRUNCATED':       _SET_BARCODE_TYPE(76),
    'GS1 DATABAR LIMITED':         _SET_BARCODE_TYPE(77),
    'GS1 DATABAR EXPANDED':        _SET_BARCODE_TYPE(78),
}

BARCODE_TYPES = {
    'A': BARCODE_TYPE_A,
    'B': BARCODE_TYPE_B,
}

# QRCode error correction levels
QR_ECLEVEL_L = 0
QR_ECLEVEL_M = 1
QR_ECLEVEL_Q = 2
QR_ECLEVEL_H = 3

# QRcode models
QR_MODEL_1 = 1
QR_MODEL_2 = 2
QR_MICRO = 3

# Image format
# NOTE: _PRINT_RASTER_IMG is the obsolete ESC/POS "print raster bit image"
#       command. The constants include a fragment of the data's header.
_PRINT_RASTER_IMG = lambda data: GS + b'v0' + data
S_RASTER_N  = _PRINT_RASTER_IMG(b'\x00')  # Set raster image normal size
S_RASTER_2W = _PRINT_RASTER_IMG(b'\x01')  # Set raster image double width
S_RASTER_2H = _PRINT_RASTER_IMG(b'\x02')  # Set raster image double height
S_RASTER_Q  = _PRINT_RASTER_IMG(b'\x03')  # Set raster image quadruple

# Printing Density
PD_N50 = GS + b'\x7c\x00'  # Printing Density -50%
PD_N37 = GS + b'\x7c\x01'  # Printing Density -37.5%
PD_N25 = GS + b'\x7c\x02'  # Printing Density -25%
PD_N12 = GS + b'\x7c\x03'  # Printing Density -12.5%
PD_0   = GS + b'\x7c\x04'  # Printing Density  0%
PD_P50 = GS + b'\x7c\x08'  # Printing Density +50%
PD_P37 = GS + b'\x7c\x07'  # Printing Density +37.5%
PD_P25 = GS + b'\x7c\x06'  # Printing Density +25%
PD_P12 = GS + b'\x7c\x05'  # Printing Density +12.5%

COLS = 56
SEPARATOR = ("=" * COLS)
SEPARATOR_DASH = ("_" * COLS)

TOTEM_SPACING = b'\x1b\x63\x38\x02'
TOTEM_PAPER_PART_CUT = b'\x1b\x63\x39\x03'
TOTEM_PAPER_FEED = b'\x1b\x63\x30\x00'



from helper import ExtendedEnum

from ._LoyaltyToken import LoyaltyToken


class ListenedToken(ExtendedEnum):
    TK_LOYALTY_GET_LOYALTY_ID = LoyaltyToken.create_token(number="1")
    TK_LOYALTY_GET_AND_LOCK_VOUCHER = LoyaltyToken.create_token(number="2")
    TK_LOYALTY_BURN_VOUCHER = LoyaltyToken.create_token(number="3")
    TK_LOYALTY_UNLOCK_VOUCHER = LoyaltyToken.create_token(number="4")
    TK_LOYALTY_GET_ID_TYPE = LoyaltyToken.create_token(number="5")
    TK_LOYALTY_LOCK_BENEFIT = LoyaltyToken.create_token(number="6")
    TK_LOYALTY_GET_ENDPOINTS_CONFIG = LoyaltyToken.create_token(number="7")
    TK_LOYALTY_GET_PAYMENT_BENEFIT = LoyaltyToken.create_token(number="8")
    TK_LOYALTY_GET_TOTALED_MESSAGE = LoyaltyToken.create_token(number="9")

    TK_LOYALTY_IS_AVAILABLE = LoyaltyToken.create_token(number="FF")



from typing import Optional


class PaymentPicture(object):
    def __init__(
        self,
        store_id,             # type: str
        payment_info=None,    # type: Optional[dict]
    ):
        # type: (...) -> None

        self.store_code = store_id
        self.payment_info = payment_info



# nn.py - function for handling Belgian national numbers
# coding: utf-8
#
# Copyright (C) 2021-2022 Cédric Krier
# Copyright (C) 2023 Jeff Horemans
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""NN, NISS, RRN (Belgian national number).

The national registration number (Rijksregisternummer, Numéro de registre
national, Nationalregisternummer) is a unique identification number of
natural persons who are registered in Belgium.

The number consists of 11 digits and includes the person's date of birth and
gender. It encodes the date of birth in the first 6 digits in the format
YYMMDD. The following 3 digits represent a counter of people born on the same
date, seperated by sex (odd for male and even for females respectively). The
final 2 digits form a check number based on the 9 preceding digits.

Special cases include:

* Counter exhaustion:
  When the even or uneven day counter range for a specific date of birth runs
  out, (e.g. from 001 tot 997 for males), the first 2 digits will represent
  the birth year as normal, while the next 4 digits (birth month and day) are
  taken to be zeroes. The remaining 3 digits still represent a day counter
  which will then restart.
  When those ranges would run out also, the sixth digit is incremented with 1
  and the day counter will restart again.

* Incomplete date of birth
  When the exact month or day of the birth date were not known at the time of
  assignment, incomplete parts are taken to be zeroes, similarly as with
  counter exhaustion.
  Note that a month with zeroes can thus both mean the date of birth was not
  exactly known, or the person was born on a day were at least 500 persons of
  the same gender got a number assigned already.

* Unknown date of birth
  When no part of the date of birth was known, a fictitious date is used
  depending on the century (i.e. 1900/00/01 or 2000/00/01).

More information:

* https://nl.wikipedia.org/wiki/Rijksregisternummer
* https://fr.wikipedia.org/wiki/Numéro_de_registre_national
* https://www.ibz.rrn.fgov.be/fileadmin/user_upload/nl/rr/instructies/IT-lijst/IT000_Rijksregisternummer.pdf

>>> compact('85.07.30-033 28')
'85073003328'
>>> validate('85 07 30 033 28')
'85073003328'
>>> validate('17 07 30 033 84')
'17073003384'
>>> validate('12345678901')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> format('85073003328')
'85.07.30-033.28'
>>> get_birth_date('85.07.30-033 28')
datetime.date(1985, 7, 30)
>>> get_birth_year('85.07.30-033 28')
1985
>>> get_birth_month('85.07.30-033 28')
7
>>> get_gender('85.07.30-033 28')
'M'
"""

import calendar
import datetime

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation. This strips the number
    of any valid separators and removes surrounding whitespace."""
    number = clean(number, ' -.').strip()
    return number


def _checksum(number):
    """Calculate the checksum and return the detected century."""
    numbers = [number]
    if int(number[:2]) + 2000 <= datetime.date.today().year:
        numbers.append('2' + number)
    for century, n in zip((1900, 2000), numbers):
        if 97 - (int(n[:-2]) % 97) == int(n[-2:]):
            return century
    raise InvalidChecksum()


def _get_birth_date_parts(number):
    """Check if the number's encoded birth date is valid, and return the contained
    birth year, month and day of month, accounting for unknown values."""
    century = _checksum(number)

    # If the fictitious dates 1900/00/01 or 2000/00/01 are detected,
    # the birth date (including the year) was not known when the number
    # was issued.
    if number[:6] in ('000001', '002001', '004001'):
        return (None, None, None)

    year = int(number[:2]) + century
    month = int(number[2:4]) % 20
    day = int(number[4:6])
    # When the month is zero, it was either unknown when the number was issued,
    # or the day counter ran out. In both cases, the month and day are not known
    # reliably.
    if month == 0:
        return (year, None, None)

    # Verify range of month
    if month > 12:
        raise InvalidComponent('Month must be in 1..12')

    # Case when only the day of the birth date is unknown
    if day == 0 or day > calendar.monthrange(year, month)[1]:
        return (year, month, None)

    return (year, month, day)


def validate(number):
    """Check if the number is a valid National Number."""
    number = compact(number)
    if not isdigits(number) or int(number) <= 0:
        raise InvalidFormat()
    if len(number) != 11:
        raise InvalidLength()
    _get_birth_date_parts(number)
    if not 0 <= int(number[2:4]) <= 12:
        raise InvalidComponent('Month must be in 1..12')
    return number


def is_valid(number):
    """Check if the number is a valid National Number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number):
    """Reformat the number to the standard presentation format."""
    number = compact(number)
    return (
        '.'.join(number[i:i + 2] for i in range(0, 6, 2)) +
        '-' + '.'.join([number[6:9], number[9:11]]))


def get_birth_year(number):
    """Return the year of the birth date."""
    year, month, day = _get_birth_date_parts(compact(number))
    return year


def get_birth_month(number):
    """Return the month of the birth date."""
    year, month, day = _get_birth_date_parts(compact(number))
    return month


def get_birth_date(number):
    """Return the date of birth."""
    year, month, day = _get_birth_date_parts(compact(number))
    if None not in (year, month, day):
        return datetime.date(year, month, day)


def get_gender(number):
    """Get the person's gender ('M' or 'F')."""
    number = compact(number)
    if int(number[6:9]) % 2:
        return 'M'
    return 'F'



# -*- coding: utf-8 -*-
import json
from logging import Logger
from threading import Condition

from messagebus import MessageBus, Event, AckMessage, NakMessage
from messageprocessorutil import UuidMessageProcessor
from messagetoken import EventToken


ELECTRONIC_PAYMENT = "ELECTRONIC_PAYMENT"
DEFAULT_TITLE = "$ELECTRONIC_PAYMENT"


class DialogManager(UuidMessageProcessor):
    def parse_data(self, data, event_type):
        return data.split('|')

    def call_business(self, obj):
        self.logger.debug("obj = {}".format(obj))
        pos_id, response = obj
        if pos_id not in self._message_conditions:
            return
        cond = self._message_conditions[pos_id]
        cond.acquire()
        self._dialog_response = response
        cond.notify_all()
        cond.release()

    def format_response(self, message_bus, message, event, input_obj, result):
        message_bus.reply_message(message, AckMessage())

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, NakMessage(exception.message))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, NakMessage(exception.message))

    def __init__(self, message_bus, logger):
        # type: (MessageBus, Logger) -> None
        super(DialogManager, self).__init__()
        self.logger = logger
        self._message_bus = message_bus
        self._message_conditions = {}
        self._dialog_response = None

    def show_message(self, pos_id, message, title=DEFAULT_TITLE):
        if not message:
            return

        data = "{}\\0{}\\0{}".format(pos_id, message, title)
        self._message_bus.publish_event(Event(EventToken.SHOW_MESSAGE, ELECTRONIC_PAYMENT, data))

    def show_question(self, pos_id, message, title=DEFAULT_TITLE, buttons="$YES|$NO"):
        self._message_bus.publish_event(Event(subject=EventToken.SHOW_QUESTION, event_type=ELECTRONIC_PAYMENT,
                                              data="{}\\0{}\\0{}\\0{}".format(pos_id, message, title, buttons)))
        self._wait_condition(pos_id)
        return self._dialog_response

    def show_choices(self, pos_id, choices, message, title=DEFAULT_TITLE):
        self._message_bus.publish_event(Event(subject=EventToken.SHOW_CHOICES, event_type=ELECTRONIC_PAYMENT,
                                              data="{}\\0{}\\0{}\\0{}".format(pos_id, message, title, json.dumps(choices))))
        self._wait_condition(pos_id)
        return self._dialog_response

    def show_input(self, pos_id, message, title=DEFAULT_TITLE, input_type="text"):
        self._message_bus.publish_event(Event(subject=EventToken.SHOW_INPUT, event_type=ELECTRONIC_PAYMENT,
                                              data="{}\\0{}\\0{}\\0{}".format(pos_id, message, title, input_type)))
        self._wait_condition(pos_id)
        return self._dialog_response

    def show_qrcode(self, pos_id, code_data, title=DEFAULT_TITLE):
        event = Event(
            subject=EventToken.SHOW_QRCODE,
            event_type=ELECTRONIC_PAYMENT,
            data="{}\\0{}\\0{}".format(pos_id, code_data, title)
        )
        self._message_bus.publish_event(event)

    def show_numpad(self, pos_id, message, title=DEFAULT_TITLE):
        self._message_bus.publish_event(Event(subject=EventToken.SHOW_NUMPAD, event_type=ELECTRONIC_PAYMENT,
                                              data="{}\\0{}\\0{}".format(pos_id, message, title)))
        self._wait_condition(pos_id)
        return self._dialog_response

    def close_dialogs(self, pos_id):
        event = Event(
            subject=EventToken.CLOSE_DIALOGS,
            event_type=ELECTRONIC_PAYMENT,
            data=str(pos_id)
        )
        self._message_bus.publish_event(event)

    def close_qrcode(self, pos_id):
        event = Event(
            subject=EventToken.CLOSE_QRCODE,
            event_type=ELECTRONIC_PAYMENT,
            data=str(pos_id)
        )
        self._message_bus.publish_event(event)

    def _wait_condition(self, pos_id):
        pos_id = str(pos_id)
        if pos_id not in self._message_conditions:
            self._message_conditions[pos_id] = Condition()

        self._message_conditions[pos_id].acquire()
        self._message_conditions[pos_id].wait()
        self._message_conditions[pos_id].release()



# -*- coding: utf-8 -*-
import json

import sysactions

from actions.util import get_all_tender_types


@sysactions.action
def get_selected_other_tender(
    pos_id,             # type: str
    tender_options,     # type: str
):
    # type: (...) -> str

    tender_types = get_all_tender_types()
    tender_options = tender_options.split(',')
    tender_types = [tender for tender in tender_types if str(tender["id"]) in tender_options]
    tender_options = [tender["descr"] for tender in tender_types]
    selected_option = sysactions.show_listbox(
        posid=pos_id,
        options=tender_options,
    )
    if selected_option is None:
        return "{}"

    selected_tender = tender_types[selected_option]

    return json.dumps({
        "id": selected_tender.get("id"),
        "electronicTypeId": selected_tender.get("electronicTypeId"),
    })



import os

import cfgtools

can_cancel_payment_on_error = None


def can_cancel_tef_payment_on_error():
    # type: (...) -> str

    global can_cancel_payment_on_error

    if can_cancel_payment_on_error is None:
        config = cfgtools.read(os.environ["LOADERCFG"])
        can_cancel = config.find_value("Customizations.CanCancelTEFPaymentOnError", "false").lower()
        can_cancel_payment_on_error = can_cancel == "true"

    return can_cancel_payment_on_error



# meid.py - functions for handling Mobile Equipment Identifiers (MEIDs)
#
# Copyright (C) 2010-2017 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""MEID (Mobile Equipment Identifier).

The Mobile Equipment Identifier is used to identify a physical piece of
CDMA mobile station equipment.

>>> validate('AF 01 23 45 0A BC DE C')
'AF0123450ABCDE'
>>> validate('29360 87365 0070 3710 0')
'AF0123450ABCDE'
>>> validate('29360 87365 0070 3710 0', strip_check_digit=False)
'AF0123450ABCDEC'
>>> validate('29360 87365 0070 3710 1')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> format('af0123450abcDEC', add_check_digit=True)
'AF 01 23 45 0A BC DE C'
>>> format('af0123450abcDEC', format='dec', add_check_digit=True)
'29360 87365 0070 3710 0'
"""

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


_hex_alphabet = '0123456789ABCDEF'


def _cleanup(number):
    """Remove any grouping information from the number and removes surrounding
    whitespace."""
    return clean(number, ' -').strip().upper()


def _ishex(number):
    for x in number:
        if x not in _hex_alphabet:
            return False
    return True


def _parse(number):
    number = _cleanup(number)
    if len(number) in (14, 15):
        # 14 or 15 digit hex representation
        if not _ishex(number):
            raise InvalidFormat()
        return number[0:14], number[14:]
    elif len(number) in (18, 19):
        # 18-digit decimal representation
        if not isdigits(number):
            raise InvalidFormat()
        return number[0:18], number[18:]
    else:
        raise InvalidLength()


def calc_check_digit(number):
    """Calculate the check digit for the number. The number should not
    already have a check digit."""
    # both the 18-digit decimal format and the 14-digit hex format
    # containing only decimal digits should use the decimal Luhn check
    from stdnum import luhn
    if isdigits(number):
        return luhn.calc_check_digit(number)
    else:
        return luhn.calc_check_digit(number, alphabet=_hex_alphabet)


def compact(number, strip_check_digit=True):
    """Convert the MEID number to the minimal (hexadecimal) representation.
    This strips grouping information, removes surrounding whitespace and
    converts to hexadecimal if needed. If the check digit is to be preserved
    and conversion is done a new check digit is recalculated."""
    # first parse the number
    number, cd = _parse(number)
    # strip check digit if needed
    if strip_check_digit:
        cd = ''
    # convert to hex if needed
    if len(number) == 18:
        number = '%08X%06X' % (int(number[0:10]), int(number[10:18]))
        if cd:
            cd = calc_check_digit(number)
    # put parts back together again
    return number + cd


def _bit_length(n):
    """Return the number of bits necessary to store the number in binary."""
    try:
        return n.bit_length()
    except AttributeError:  # pragma: no cover (Python 2.6 only)
        import math
        return int(math.log(n, 2)) + 1


def validate(number, strip_check_digit=True):
    """Check if the number is a valid MEID number. This converts the
    representation format of the number (if it is decimal it is not converted
    to hexadecimal)."""
    # first parse the number
    number, cd = _parse(number)
    from stdnum import luhn
    if len(number) == 18:
        # decimal format can be easily determined
        if cd:
            luhn.validate(number + cd)
        # convert to hex
        manufacturer_code = int(number[0:10])
        serial_num = int(number[10:18])
        if _bit_length(manufacturer_code) > 32 or _bit_length(serial_num) > 24:
            raise InvalidComponent()
        number = '%08X%06X' % (manufacturer_code, serial_num)
        cd = calc_check_digit(number)
    elif isdigits(number):
        # if the remaining hex format is fully decimal it is an IMEI number
        from stdnum import imei
        imei.validate(number + cd)
    else:
        # normal hex Luhn validation
        if cd:
            luhn.validate(number + cd, alphabet=_hex_alphabet)
    if strip_check_digit:
        cd = ''
    return number + cd


def is_valid(number):
    """Check if the number is a valid MEID number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number, separator=' ', format=None, add_check_digit=False):
    """Reformat the number to the standard presentation format. The separator
    used can be provided. If the format is specified (either 'hex' or 'dec')
    the number is reformatted in that format, otherwise the current
    representation is kept. If add_check_digit is True a check digit will be
    added if it is not present yet."""
    # first parse the number
    number, cd = _parse(number)
    # format conversions if needed
    if format == 'dec' and len(number) == 14:
        # convert to decimal
        number = '%010d%08d' % (int(number[0:8], 16), int(number[8:14], 16))
        if cd:
            cd = calc_check_digit(number)
    elif format == 'hex' and len(number) == 18:
        # convert to hex
        number = '%08X%06X' % (int(number[0:10]), int(number[10:18]))
        if cd:
            cd = calc_check_digit(number)
    # see if we need to add a check digit
    if add_check_digit and not cd:
        cd = calc_check_digit(number)
    # split number according to format
    if len(number) == 14:
        number = [number[i * 2:i * 2 + 2]
                  for i in range(7)] + [cd]
    else:
        number = (number[:5], number[5:10], number[10:14], number[14:], cd)
    return separator.join(x for x in number if x)


def to_binary(number):
    """Convert the number to its binary representation (without the check
    digit)."""
    from binascii import a2b_hex
    return a2b_hex(compact(number, strip_check_digit=True))


def to_pseudo_esn(number):
    """Convert the provided MEID to a pseudo ESN (pESN). The ESN is returned
    in compact hexadecimal representation."""
    # return the last 6 digits of the SHA1  hash prefixed with the reserved
    # manufacturer code
    import hashlib
    return '80' + hashlib.sha1(to_binary(number)).hexdigest()[-6:].upper()



class PaymentInfo(object):
    def __init__(
        self,
        tender_id,          # type: int
        tender_amount,      # type: float
        tender_desc,        # type: str
        tender_type,        # type: int
        tender_detail,      # type: str
        change_amount,      # type: float
    ):
        self.tender_id = tender_id
        self.tender_amount = tender_amount
        self.tender_desc = tender_desc
        self.tender_type = tender_type
        self.tender_detail = tender_detail
        self.change_amount = change_amount


class GetBenefitOnPaymentRequest(object):
    def __init__(
        self,                   # type: GetBenefitOnPaymentRequest
        pos_id,                 # type: int
        loyalty_customer_id,    # type: str
        payment_info,           # type: PaymentInfo
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.loyalty_customer_id = loyalty_customer_id
        self.payment_info = payment_info



from ._CatalogUpdaterInteractor import CatalogUpdaterInteractor
from ._MediaUpdaterInteractor import MediaUpdaterInteractor
from ._UserUpdaterInteractor import UserUpdaterInteractor
from ._GetCatalogVersionToApplyInteractor import GetCatalogVersionToApplyInteractor
from ._LoaderUpdaterInteractor import LoaderUpdaterInteractor
from ._GetVersionsInteractor import GetVersionsInteractor
from ._BinaryUpdaterInteractor import BinaryUpdaterInteractor
from ._ChromeUpdaterInteractor import ChromeUpdaterInteractor



import json
from typing import (
    Any,
    Dict,
    List,
    Optional,
)

import persistence
from systools import sys_log_exception

from .. import mb_context

from application.domain import TenderIdEnum


def get_payments_amount_by_type(
    pos_id,             # type: str
    session_id,         # type: str
    payments_amount,    # type: Dict[int, List[Dict]]
):
    # type: (...) -> None

    conn = None

    try:
        conn = persistence.Driver().open(mb_context, str(pos_id))
        _all_non_electronic_payments(
            conn,
            session_id,
            payments_amount,
        )
        _all_electronic_payments(
            conn,
            session_id,
            payments_amount,
        )
    except:  # noqa
        sys_log_exception("Error getting payments amount by type")
        raise
    finally:
        if conn:
            conn.close()


def _all_electronic_payments(
    conn,               # type: Any
    session_id,         # type: str
    payments_amount,    # type: Dict[int, List[Dict]]
):
    # type: (...) -> None

    cursor = conn.select("""
        SELECT
            ot.TenderId, 
            ot.TenderAmount - COALESCE(ot.ChangeAmount, 0) AS Amount,
            ot.TenderDetail
        FROM OrderTender ot
        INNER JOIN Orders o ON o.OrderId = ot.OrderId
        WHERE 
            o.SessionId = '{0}' 
            AND o.StateId = 5 
            AND ot.ElectronicType NOT IN (0, 9) 
            AND ot.OrderId NOT IN (
                SELECT DISTINCT ocp.OrderId 
                FROM OrderCustomProperties ocp
                INNER JOIN Orders o ON ocp.OrderId = o.OrderId
                WHERE ocp.Key = 'DELIVERY_ORDER_MANUAL_CONFIRMED' AND o.SessionId = '{0}'
            )
    """.format(session_id))

    if cursor.rows() > 0:
        for row in cursor:
            tender_id = int(row.get_entry("TenderId"))
            tender_amount = float(row.get_entry("Amount") or 0)
            tender_detail = json.loads(row.get_entry("TenderDetail") or "{}")
            payment = _find_or_insert_payment(
                payments_amount,
                tender_id,
                tender_type=_map_or_none(tender_detail.get("tenderType")),
                card_brand=tender_detail.get("Bandeira"),
            )
            payment["amount"] += tender_amount


def _all_non_electronic_payments(
    conn,               # type: Any
    session_id,         # type: str
    payments_amount,    # type: Dict[int, List[Dict]]
):
    # type: (...) -> None

    cursor = conn.select("""
        SELECT ot.TenderId, SUM(ot.TenderAmount - COALESCE(ot.ChangeAmount, 0)) AS Amount
        FROM OrderTender ot
        INNER JOIN Orders o ON o.OrderId = ot.OrderId
        WHERE 
            SessionId = '{0}' 
            AND StateId = 5 
            AND ot.ElectronicType IN (0, 9)
            AND ot.OrderId NOT IN (
                SELECT DISTINCT ocp.OrderId
                FROM OrderCustomProperties ocp
                INNER JOIN Orders o ON ocp.OrderId = o.OrderId
                WHERE ocp.Key = 'DELIVERY_ORDER_MANUAL_CONFIRMED' AND o.SessionId = '{0}'
            )
        GROUP BY ot.TenderId
    """.format(session_id))

    if cursor.rows() > 0:
        for row in cursor:
            tender_id = int(row.get_entry("TenderId"))
            tender_amount = float(row.get_entry("Amount") or 0)
            payment = _find_or_insert_payment(
                payments_amount,
                tender_id,
            )
            payment["amount"] += tender_amount


def _find_or_insert_payment(payments, tender_id, tender_type=None, card_brand=None):
    payment_list = payments.setdefault(tender_id, [])
    for payment in payment_list:
        same_tender_type = tender_type == payment.get("tenderType")
        same_card_brand = card_brand == payment.get("cardBrand")
        if same_tender_type and same_card_brand:
            return payment

    default_payment = {
        "amount": 0.0,
        "linked": False,
        "tenderType": tender_type,
        "cardBrand": card_brand,
        "tenderId": tender_id,
    }
    payment_list.append(default_payment)
    return default_payment


def _map_or_none(
    tender_type,  # type: Optional[str]
):
    # type: (...) -> Optional[str]

    if tender_type is not None:
        return _map_tender_type(str(tender_type))
    return None


def _map_tender_type(
    tender_type,  # type: str
):
    # type: (...) -> str

    types_map = {
        str(TenderIdEnum.CREDIT_CARD.value): "03",
        str(TenderIdEnum.DEBIT_CARD.value): "04",
    }
    return types_map.get(tender_type, tender_type)



from abc import ABCMeta

from messagebus import AckMessage, NakMessage
from messageprocessorutil import UuidMessageProcessor
from paymentrouter import PaymentRouter


class _CommonProcessor(UuidMessageProcessor):
    __metaclass__ = ABCMeta

    def __init__(self, payment_router):
        # type: (PaymentRouter) -> None
        super(_CommonProcessor, self).__init__()
        self.payment_router = payment_router

    def format_response(self, message_bus, message, event, input_obj, result):
        message_bus.reply_message(message, AckMessage((result or "")))

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, NakMessage(exception.message))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, NakMessage(exception.message))



# btw.py - functions for handling Dutch VAT numbers
#
# Copyright (C) 2012-2020 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Btw-identificatienummer (Omzetbelastingnummer, the Dutch VAT number).

The btw-identificatienummer (previously the btw-nummer) is the Dutch number
for identifying parties in a transaction for which VAT is due. The btw-nummer
is used in communication with the tax agency while the
btw-identificatienummer (EORI-nummer) can be used when dealing with other
companies though they are used interchangeably.

The btw-nummer consists of a RSIN or BSN followed by the letter B and two
digits that identify the number of the company created. The
btw-identificatienummer has a similar format but different checksum and does
not contain the BSN.

More information:

* https://en.wikipedia.org/wiki/VAT_identification_number
* https://nl.wikipedia.org/wiki/Btw-nummer_(Nederland)

>>> validate('004495445B01')
'004495445B01'
>>> validate('NL4495445B01')
'004495445B01'
>>> validate('NL002455799B11')  # valid since 2020-01-01
'002455799B11'
>>> validate('123456789B90')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
"""

from stdnum.exceptions import *
from stdnum.iso7064 import mod_97_10
from stdnum.nl import bsn
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    number = clean(number, ' -.').upper().strip()
    if number.startswith('NL'):
        number = number[2:]
    return bsn.compact(number[:-3]) + number[-3:]


def validate(number):
    """Check if the number is a valid btw number. This checks the length,
    formatting and check digit."""
    number = compact(number)
    if not isdigits(number[:9]) or int(number[:9]) <= 0:
        raise InvalidFormat()
    if not isdigits(number[10:]) or int(number[10:]) <= 0:
        raise InvalidFormat()
    if len(number) != 12:
        raise InvalidLength()
    if number[9] != 'B':
        raise InvalidFormat()
    if not bsn.is_valid(number[:9]) and not mod_97_10.is_valid('NL' + number):
        raise InvalidChecksum()
    return number


def is_valid(number):
    """Check if the number is a valid btw number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False



from production.box import StampItemsViewBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class StampItemsViewBoxBuilder(BoxBuilder):
    def __init__(self, order_changer, views, loggers):
        super(StampItemsViewBoxBuilder, self).__init__(loggers)
        self.order_changer = order_changer
        self.views = views
        self.loggers = loggers

    def build(self, config):
        # type: (BoxConfiguration) -> StampItemsViewBox
        return StampItemsViewBox(
            config.name,
            self.order_changer,
            self.views[config.extra_config["View"]],
            config.sons,
            self.get_logger(config)
        )



# -*- coding: utf-8 -*-

import sysactions

from bustoken import TK_POS_UPDATER_PERFORM_MEDIA_UPDATE
from msgbus import TK_SYS_NAK, FM_PARAM

from actions.update.update_util import show_nak_message


@sysactions.action
def update_media(pos_id):
    msg = sysactions.send_message("PosUpdater", TK_POS_UPDATER_PERFORM_MEDIA_UPDATE, FM_PARAM, timeout=60000000 * 5)
    if msg.token == TK_SYS_NAK:
        if not msg.data:
            sysactions.show_messagebox(pos_id, "$UPDATE_MEDIA_DISABLED" + msg.data.upper(), buttons="OK")
            return
        else:
            response_message = msg.data
            show_nak_message(pos_id, "$UPDATE_MEDIA_", response_message)

    sysactions.show_messagebox(pos_id, "$UPDATE_MEDIA_MESSAGE", buttons="OK")



from typing import Optional


class FeeDetails(object):

    def __init__(
        self,                   # type: FeeDetails
        code,                   # type: str
        product_code=None,      # type: Optional[int]
        product_name=None,      # type: Optional[str]
    ):
        # type: (...) -> None

        self.code = code
        self.product_code = product_code
        self.product_name = product_name



from typing import Any

from application.interactor import BinaryUpdaterInteractor
from application.model import AsyncUuidMessageProcessor
from messagebus import DefaultToken, DataType, Message
from application.util import settings


class BinaryUpdaterProcessor(AsyncUuidMessageProcessor):
    def __init__(self, interactor):
        # type: (BinaryUpdaterInteractor) -> None

        super(BinaryUpdaterProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(self, data, event_type):
        # type: (Any, Any) -> Any

        return data or self._do_not_remove_this_function_bug_in_the_library()

    def call_business(self, data):
        # type: (Any) -> Any

        settings.clear_status()
        return self.interactor.update_binaries(data)

    def format_response(self, message_bus, message, event, _, response):
        reply_message = Message(DefaultToken.TK_SYS_ACK, DataType.param, response)
        if message:
            message_bus.reply_message(message, reply_message)
        elif event:
            message_bus.reply_event(event, reply_message)

    def format_exception(self, message_bus, message, event, _, exception):
        self._reply_exception(event, exception, message, message_bus)

    def format_parse_exception(self, message_bus, message, event, _, exception):
        self._reply_exception(event, exception, message, message_bus)

    @staticmethod
    def _reply_exception(event, exception, message, message_bus):
        reply_message = Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception))
        if message:
            reply_message = Message(DefaultToken.TK_SYS_NAK, DataType.param, str(type(exception).__name__))
            message_bus.reply_message(message, reply_message)
        elif event:
            message_bus.reply_event(event, reply_message)

    @staticmethod
    def _do_not_remove_this_function_bug_in_the_library():
        return ''



import json

from logging import Logger
from xml.etree import cElementTree as eTree

from application.model.customexception import (
    BenefitIdNotFoundInAppliedOrderBenefitsException,
    OrderHasNotBenefitListException,
)


def get_applier_sub_type_from_order(
    benefit_id,     # type: str
    order_picture,  # type: str
    logger,         # type: Logger
):
    # type: (...) -> str

    try:
        order = eTree.XML(order_picture)
        order_id = order.get("orderId")
        benefit_list = order.find(".//CustomOrderProperties/OrderProperty[@key='BENEFIT_LIST']")
        if benefit_list is None:
            raise OrderHasNotBenefitListException(
                order_id=order_id,
            )

        benefit_list = json.loads(benefit_list.get("value"))
        benefits = json.loads(benefit_list.get("benefitList", "{}"))

        for benefit in benefits:
            benefit_info = benefit.get("benefit", {})
            if benefit_info.get("id") != benefit_id:
                continue

            applier_sub_type = benefit_info.get("applierSubType")
            if applier_sub_type:
                return applier_sub_type

            break

        raise BenefitIdNotFoundInAppliedOrderBenefitsException(
            order_id=order_id,
            benefit_id=benefit_id,
        )

    except Exception as ex:
        logger.exception("Error trying to get applier sub type for benefitId: {}".format(benefit_id))
        raise ex



from application.model.configuration import DefaultManagerConfig


class MediaConfigurations(DefaultManagerConfig):
    def __init__(self):

        super(MediaConfigurations, self).__init__()
        self.images_directory = None
        self.endpoints = Endpoints()


class Endpoints(object):
    def __init__(self):
        self.get_download_image_permission = None
        self.get_download_image_totem = None
